package sun.text.normalizer;

import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.BufferedInputStream;

public final class NormalizerImpl
{
    static final NormalizerImpl IMPL;
    static final int UNSIGNED_BYTE_MASK = 255;
    static final long UNSIGNED_INT_MASK = 4294967295L;
    private static final String DATA_FILE_NAME = "/sun/text/resources/unorm.icu";
    public static final int QC_NFC = 17;
    public static final int QC_NFKC = 34;
    public static final int QC_NFD = 4;
    public static final int QC_NFKD = 8;
    public static final int QC_ANY_NO = 15;
    public static final int QC_MAYBE = 16;
    public static final int QC_ANY_MAYBE = 48;
    public static final int QC_MASK = 63;
    private static final int COMBINES_FWD = 64;
    private static final int COMBINES_BACK = 128;
    public static final int COMBINES_ANY = 192;
    private static final int CC_SHIFT = 8;
    public static final int CC_MASK = 65280;
    private static final int EXTRA_SHIFT = 16;
    private static final long MIN_SPECIAL = 4227858432L;
    private static final long SURROGATES_TOP = 4293918720L;
    private static final long MIN_HANGUL = 4293918720L;
    private static final long JAMO_V_TOP = 4294115328L;
    static final int INDEX_TRIE_SIZE = 0;
    static final int INDEX_CHAR_COUNT = 1;
    static final int INDEX_COMBINE_DATA_COUNT = 2;
    public static final int INDEX_MIN_NFC_NO_MAYBE = 6;
    public static final int INDEX_MIN_NFKC_NO_MAYBE = 7;
    public static final int INDEX_MIN_NFD_NO_MAYBE = 8;
    public static final int INDEX_MIN_NFKD_NO_MAYBE = 9;
    static final int INDEX_FCD_TRIE_SIZE = 10;
    static final int INDEX_AUX_TRIE_SIZE = 11;
    static final int INDEX_TOP = 32;
    private static final int AUX_UNSAFE_SHIFT = 11;
    private static final int AUX_COMP_EX_SHIFT = 10;
    private static final int AUX_NFC_SKIPPABLE_F_SHIFT = 12;
    private static final int AUX_MAX_FNC = 1024;
    private static final int AUX_UNSAFE_MASK = 2048;
    private static final int AUX_FNC_MASK = 1023;
    private static final int AUX_COMP_EX_MASK = 1024;
    private static final long AUX_NFC_SKIP_F_MASK = 4096L;
    private static final int MAX_BUFFER_SIZE = 20;
    private static NormalizerImpl.FCDTrieImpl fcdTrieImpl;
    private static NormalizerImpl.NormTrieImpl normTrieImpl;
    private static NormalizerImpl.AuxTrieImpl auxTrieImpl;
    private static int[] indexes;
    private static char[] combiningTable;
    private static char[] extraData;
    private static boolean isDataLoaded;
    private static boolean isFormatVersion_2_1;
    private static boolean isFormatVersion_2_2;
    private static byte[] unicodeVersion;
    private static final int DATA_BUFFER_SIZE = 25000;
    public static final int MIN_WITH_LEAD_CC = 768;
    private static final int DECOMP_FLAG_LENGTH_HAS_CC = 128;
    private static final int DECOMP_LENGTH_MASK = 127;
    private static final int BMP_INDEX_LENGTH = 2048;
    private static final int SURROGATE_BLOCK_BITS = 5;
    public static final int JAMO_L_BASE = 4352;
    public static final int JAMO_V_BASE = 4449;
    public static final int JAMO_T_BASE = 4519;
    public static final int HANGUL_BASE = 44032;
    public static final int JAMO_L_COUNT = 19;
    public static final int JAMO_V_COUNT = 21;
    public static final int JAMO_T_COUNT = 28;
    public static final int HANGUL_COUNT = 11172;
    private static final int OPTIONS_NX_MASK = 31;
    private static final int OPTIONS_UNICODE_MASK = 224;
    public static final int OPTIONS_SETS_MASK = 255;
    private static final UnicodeSet[] nxCache;
    private static final int NX_HANGUL = 1;
    private static final int NX_CJK_COMPAT = 2;
    public static final int BEFORE_PRI_29 = 256;
    public static final int OPTIONS_COMPAT = 4096;
    public static final int OPTIONS_COMPOSE_CONTIGUOUS = 8192;
    public static final int WITHOUT_CORRIGENDUM4_CORRECTIONS = 262144;
    private static final char[][] corrigendum4MappingTable;
    
    public static int getFromIndexesArr(int n) {
        return NormalizerImpl.indexes[n];
    }
    
    private NormalizerImpl() throws IOException {
        if (!NormalizerImpl.isDataLoaded) {
            BufferedInputStream bufferedInputStream = new BufferedInputStream(ICUData.getRequiredStream("/sun/text/resources/unorm.icu"), 25000);
            NormalizerDataReader normalizerDataReader = new NormalizerDataReader(bufferedInputStream);
            NormalizerImpl.indexes = normalizerDataReader.readIndexes(32);
            byte[] buf = new byte[NormalizerImpl.indexes[0]];
            NormalizerImpl.combiningTable = new char[NormalizerImpl.indexes[2]];
            NormalizerImpl.extraData = new char[NormalizerImpl.indexes[1]];
            byte[] buf2 = new byte[NormalizerImpl.indexes[10]];
            byte[] buf3 = new byte[NormalizerImpl.indexes[11]];
            NormalizerImpl.fcdTrieImpl = new NormalizerImpl.FCDTrieImpl();
            NormalizerImpl.normTrieImpl = new NormalizerImpl.NormTrieImpl();
            NormalizerImpl.auxTrieImpl = new NormalizerImpl.AuxTrieImpl();
            normalizerDataReader.read(buf, buf2, buf3, NormalizerImpl.extraData, NormalizerImpl.combiningTable);
            NormalizerImpl.NormTrieImpl.normTrie = new IntTrie(new ByteArrayInputStream(buf), NormalizerImpl.normTrieImpl);
            NormalizerImpl.FCDTrieImpl.fcdTrie = new CharTrie(new ByteArrayInputStream(buf2), NormalizerImpl.fcdTrieImpl);
            NormalizerImpl.AuxTrieImpl.auxTrie = new CharTrie(new ByteArrayInputStream(buf3), NormalizerImpl.auxTrieImpl);
            NormalizerImpl.isDataLoaded = true;
            byte[] dataFormatVersion = normalizerDataReader.getDataFormatVersion();
            NormalizerImpl.isFormatVersion_2_1 = (dataFormatVersion[0] > 2 || (dataFormatVersion[0] == 2 && dataFormatVersion[1] >= 1));
            NormalizerImpl.isFormatVersion_2_2 = (dataFormatVersion[0] > 2 || (dataFormatVersion[0] == 2 && dataFormatVersion[1] >= 2));
            NormalizerImpl.unicodeVersion = normalizerDataReader.getUnicodeVersion();
            bufferedInputStream.close();
        }
    }
    
    private static boolean isHangulWithoutJamoT(char c) {
        int n = (char)(c - 44032);
        return n < '⮤' && n % 28 == 0;
    }
    
    private static boolean isNorm32Regular(long n) {
        return n < 4227858432L;
    }
    
    private static boolean isNorm32LeadSurrogate(long n) {
        return 4227858432L <= n && n < 4293918720L;
    }
    
    private static boolean isNorm32HangulOrJamo(long n) {
        return n >= 4293918720L;
    }
    
    private static boolean isJamoVTNorm32JamoV(long n) {
        return n < 4294115328L;
    }
    
    public static long getNorm32(char c) {
        return 0xFFFFFFFFL & (long)NormalizerImpl.NormTrieImpl.normTrie.getLeadValue(c);
    }
    
    public static long getNorm32FromSurrogatePair(long n, char c) {
        return 0xFFFFFFFFL & (long)NormalizerImpl.NormTrieImpl.normTrie.getTrailValue((int)n, c);
    }
    
    private static long getNorm32(int n) {
        return 0xFFFFFFFFL & (long)NormalizerImpl.NormTrieImpl.normTrie.getCodePointValue(n);
    }
    
    private static long getNorm32(char[] array, int n, int n2) {
        long n3 = getNorm32(array[n]);
        if ((n3 & (long)n2) > 0L && isNorm32LeadSurrogate(n3)) {
            n3 = getNorm32FromSurrogatePair(n3, array[n + 1]);
        }
        return n3;
    }
    
    public static VersionInfo getUnicodeVersion() {
        return VersionInfo.getInstance((int)NormalizerImpl.unicodeVersion[0], (int)NormalizerImpl.unicodeVersion[1], (int)NormalizerImpl.unicodeVersion[2], (int)NormalizerImpl.unicodeVersion[3]);
    }
    
    public static char getFCD16(char c) {
        return NormalizerImpl.FCDTrieImpl.fcdTrie.getLeadValue(c);
    }
    
    public static char getFCD16FromSurrogatePair(char c, char c2) {
        return NormalizerImpl.FCDTrieImpl.fcdTrie.getTrailValue((int)c, c2);
    }
    
    public static int getFCD16(int n) {
        return NormalizerImpl.FCDTrieImpl.fcdTrie.getCodePointValue(n);
    }
    
    private static int getExtraDataIndex(long n) {
        return (int)(n >> 16);
    }
    
    private static int decompose(long n, int n2, NormalizerImpl.DecomposeArgs decomposeArgs) {
        int extraDataIndex = getExtraDataIndex(n);
        decomposeArgs.length = NormalizerImpl.extraData[extraDataIndex++];
        if ((n & (long)n2 & 0x8L) != 0x0L && decomposeArgs.length >= 256) {
            extraDataIndex += (decomposeArgs.length >> 7 & 0x1) + (decomposeArgs.length & 0x7F);
            decomposeArgs.length >>= 8;
        }
        if ((decomposeArgs.length & 0x80) > 0) {
            int n4 = NormalizerImpl.extraData[extraDataIndex++];
            decomposeArgs.cc = (0xFF & n4 >> 8);
            decomposeArgs.trailCC = ('ÿ' & n4);
        }
        else {
            int n3 = 0;
            decomposeArgs.trailCC = n3;
            decomposeArgs.cc = n3;
        }
        decomposeArgs.length &= 0x7F;
        return extraDataIndex;
    }
    
    private static int decompose(long n, NormalizerImpl.DecomposeArgs decomposeArgs) {
        int extraDataIndex = getExtraDataIndex(n);
        decomposeArgs.length = NormalizerImpl.extraData[extraDataIndex++];
        if ((decomposeArgs.length & 0x80) > 0) {
            int n3 = NormalizerImpl.extraData[extraDataIndex++];
            decomposeArgs.cc = (0xFF & n3 >> 8);
            decomposeArgs.trailCC = ('ÿ' & n3);
        }
        else {
            int n2 = 0;
            decomposeArgs.trailCC = n2;
            decomposeArgs.cc = n2;
        }
        decomposeArgs.length &= 0x7F;
        return extraDataIndex;
    }
    
    private static int getNextCC(NormalizerImpl.NextCCArgs nextCCArgs) {
        nextCCArgs.c = nextCCArgs.source[nextCCArgs.next++];
        long n = getNorm32(nextCCArgs.c);
        if ((n & 0xFF00L) == 0x0L) {
            nextCCArgs.c2 = '\0';
            return 0;
        }
        if (!isNorm32LeadSurrogate(n)) {
            nextCCArgs.c2 = '\0';
        }
        else {
            if (nextCCArgs.next != nextCCArgs.limit) {
                char c2 = nextCCArgs.source[nextCCArgs.next];
                nextCCArgs.c2 = c2;
                if (UTF16.isTrailSurrogate(c2)) {
                    ++nextCCArgs.next;
                    n = getNorm32FromSurrogatePair(n, nextCCArgs.c2);
                    return (int)(0xFFL & n >> 8);
                }
            }
            nextCCArgs.c2 = '\0';
            return 0;
        }
        return (int)(0xFFL & n >> 8);
    }
    
    private static long getPrevNorm32(NormalizerImpl.PrevArgs prevArgs, int n, int n2) {
        char[] src = prevArgs.src;
        int current = prevArgs.current - 1;
        prevArgs.current = current;
        prevArgs.c = src[current];
        prevArgs.c2 = '\0';
        if (prevArgs.c < n) {
            return 0L;
        }
        if (!UTF16.isSurrogate(prevArgs.c)) {
            return getNorm32(prevArgs.c);
        }
        if (UTF16.isLeadSurrogate(prevArgs.c)) {
            return 0L;
        }
        if (prevArgs.current != prevArgs.start) {
            char c2 = prevArgs.src[prevArgs.current - 1];
            prevArgs.c2 = c2;
            if (UTF16.isLeadSurrogate(c2)) {
                --prevArgs.current;
                long norm32 = getNorm32(prevArgs.c2);
                if ((norm32 & (long)n2) == 0x0L) {
                    return 0L;
                }
                return getNorm32FromSurrogatePair(norm32, prevArgs.c);
            }
        }
        prevArgs.c2 = '\0';
        return 0L;
    }
    
    private static int getPrevCC(NormalizerImpl.PrevArgs prevArgs) {
        return (int)(0xFFL & getPrevNorm32(prevArgs, 768, 65280) >> 8);
    }
    
    public static boolean isNFDSafe(long n, int n2, int n3) {
        if ((n & (long)n2) == 0x0L) {
            return true;
        }
        if (isNorm32Regular(n) && (n & (long)n3) != 0x0L) {
            NormalizerImpl.DecomposeArgs decomposeArgs = new NormalizerImpl.DecomposeArgs();
            decompose(n, n3, decomposeArgs);
            return decomposeArgs.cc == 0;
        }
        return (n & 0xFF00L) == 0x0L;
    }
    
    public static boolean isTrueStarter(long n, int n2, int n3) {
        if ((n & (long)n2) == 0x0L) {
            return true;
        }
        if ((n & (long)n3) != 0x0L) {
            NormalizerImpl.DecomposeArgs decomposeArgs = new NormalizerImpl.DecomposeArgs();
            int decompose = decompose(n, n3, decomposeArgs);
            if (decomposeArgs.cc == 0) {
                int n4 = n2 & 0x3F;
                if ((getNorm32(NormalizerImpl.extraData, decompose, n4) & (long)n4) == 0x0L) {
                    return true;
                }
            }
        }
        return false;
    }
    
    private static int insertOrdered(char[] src, int i, int current, int n, char c, char c2, int n2) {
        if (i < current && n2 != 0) {
            NormalizerImpl.PrevArgs prevArgs = new NormalizerImpl.PrevArgs();
            prevArgs.current = current;
            prevArgs.start = i;
            prevArgs.src = src;
            int prevCC = getPrevCC(prevArgs);
            int n3 = prevArgs.current;
            if (n2 < prevCC) {
                n2 = prevCC;
                int j = n3;
                while (i < n3) {
                    int prevCC2 = getPrevCC(prevArgs);
                    n3 = prevArgs.current;
                    if (n2 >= prevCC2) {
                        break;
                    }
                    j = n3;
                }
                do {
                    src[--n] = src[--current];
                } while (j != current);
            }
        }
        src[current] = c;
        if (c2 != '\0') {
            src[current + 1] = c2;
        }
        return n2;
    }
    
    private static int mergeOrdered(char[] array, int start, int next, char[] array2, int next2, int limit, boolean b) {
        int insertOrdered = 0;
        int n = next == next2;
        NormalizerImpl.NextCCArgs nextCCArgs = new NormalizerImpl.NextCCArgs();
        nextCCArgs.source = array2;
        nextCCArgs.next = next2;
        nextCCArgs.limit = limit;
        if (start != next || !b) {
            while (nextCCArgs.next < nextCCArgs.limit) {
                int nextCC = getNextCC(nextCCArgs);
                if (nextCC == 0) {
                    insertOrdered = 0;
                    if (!(n == 0)) {
                        next = nextCCArgs.next;
                    }
                    else {
                        array2[next++] = nextCCArgs.c;
                        if (nextCCArgs.c2 != '\0') {
                            array2[next++] = nextCCArgs.c2;
                        }
                    }
                    if (b) {
                        break;
                    }
                    start = next;
                }
                else {
                    int n2 = next + ((nextCCArgs.c2 == '\0') ? 1 : 2);
                    insertOrdered = insertOrdered(array, start, next, n2, nextCCArgs.c, nextCCArgs.c2, nextCC);
                    next = n2;
                }
            }
        }
        if (nextCCArgs.next == nextCCArgs.limit) {
            return insertOrdered;
        }
        if (n == 0) {
            do {
                array[next++] = array2[nextCCArgs.next++];
            } while (nextCCArgs.next != nextCCArgs.limit);
            nextCCArgs.limit = next;
        }
        NormalizerImpl.PrevArgs o = new NormalizerImpl.PrevArgs();
        o.src = array2;
        o.start = start;
        o.current = nextCCArgs.limit;
        return getPrevCC(o);
    }
    
    private static int mergeOrdered(char[] array, int n, int n2, char[] array2, int n3, int n4) {
        return mergeOrdered(array, n, n2, array2, n3, n4, true);
    }
    
    public static NormalizerBase.QuickCheckResult quickCheck(char[] array, int i, int n, int n2, int n3, int n4, boolean b, UnicodeSet set) {
        NormalizerImpl.ComposePartArgs composePartArgs = new NormalizerImpl.ComposePartArgs();
        if (!NormalizerImpl.isDataLoaded) {
            return NormalizerBase.MAYBE;
        }
        int n5 = 0xFF00 | n3;
        NormalizerBase.QuickCheckResult quickCheckResult = NormalizerBase.YES;
        int prevCC = '\0';
        while (i != n) {
            int n6;
            long n7;
            if ((n6 = array[i++]) >= n2 && ((n7 = getNorm32((char)n6)) & (long)n5) != 0x0L) {
                int n8;
                if (isNorm32LeadSurrogate(n7)) {
                    if (i != n && UTF16.isTrailSurrogate((char)(n8 = array[i]))) {
                        ++i;
                        n7 = getNorm32FromSurrogatePair(n7, (char)n8);
                    }
                    else {
                        n7 = 0L;
                        n8 = '\0';
                    }
                }
                else {
                    n8 = '\0';
                }
                if (nx_contains(set, (char)n6, (char)n8)) {
                    n7 = 0L;
                }
                int n9 = (char)(int)(n7 >> 8 & 0xFFL);
                if (n9 != '\0' && n9 < prevCC) {
                    return NormalizerBase.NO;
                }
                prevCC = n9;
                long n10 = n7 & (long)n3;
                NormalizerBase.QuickCheckResult quickCheckResult2;
                if ((n10 & 0xFL) >= 1L) {
                    quickCheckResult2 = NormalizerBase.NO;
                }
                else {
                    if (n10 == 0L) {
                        continue;
                    }
                    if (b) {
                        quickCheckResult = NormalizerBase.MAYBE;
                        continue;
                    }
                    int n11 = n3 << 2 & 0xF;
                    int n12 = i - 1;
                    if (UTF16.isTrailSurrogate(array[n12])) {
                        --n12;
                    }
                    int previousStarter = findPreviousStarter(array, i, n12, n5, n11, (char)n2);
                    i = findNextStarter(array, i, n, n3, n11, (char)n2);
                    composePartArgs.prevCC = prevCC;
                    if (0 != strCompare(composePart(composePartArgs, previousStarter, array, i, n, n4, set), 0, composePartArgs.length, array, previousStarter, i, false)) {
                        quickCheckResult2 = NormalizerBase.NO;
                    }
                    else {
                        continue;
                    }
                }
                return quickCheckResult2;
            }
            else {
                prevCC = '\0';
            }
        }
        return quickCheckResult;
    }
    
    public static int decompose(char[] array, int n, int n2, char[] array2, int n3, int n4, boolean b, int[] array3, UnicodeSet set) {
        char[] array4 = new char[3];
        int n5;
        int n6;
        if (!b) {
            n5 = (char)NormalizerImpl.indexes[8];
            n6 = 4;
        }
        else {
            n5 = (char)NormalizerImpl.indexes[9];
            n6 = 8;
        }
        int n7 = 0xFF00 | n6;
        int n8 = 0;
        int n9 = 0;
        long n10 = 0L;
        int n11 = '\0';
        int decompose = 0;
        int cc;
        int n12 = cc = -1;
        while (true) {
            for (; n != n2 && ((n11 = array[n]) < n5 || ((n10 = getNorm32((char)n11)) & (long)n7) == 0x0L); ++n) {
                n9 = 0;
            }
            if (n != n) {
                int n13 = n - n;
                if (n3 + n13 <= n4) {
                    System.arraycopy(array, n, array2, n3, n13);
                }
                n3 = (n8 = n3 + n13);
            }
            if (n == n2) {
                break;
            }
            ++n;
            int n14;
            int length;
            char[] extraData;
            if (isNorm32HangulOrJamo(n10)) {
                if (nx_contains(set, n11)) {
                    n14 = '\0';
                    extraData = null;
                    length = 1;
                }
                else {
                    extraData = array4;
                    decompose = 0;
                    n12 = (cc = 0);
                    int n15 = (char)(n11 - 44032);
                    n14 = (char)(n15 % 28);
                    n11 = (char)(n15 / 28);
                    if (n14 > '\0') {
                        array4[2] = (char)(4519 + n14);
                        length = 3;
                    }
                    else {
                        length = 2;
                    }
                    array4[1] = (char)(4449 + n11 % 21);
                    array4[0] = (char)(4352 + n11 / 21);
                }
            }
            else {
                if (isNorm32Regular(n10)) {
                    n14 = '\0';
                    length = 1;
                }
                else if (n != n2 && UTF16.isTrailSurrogate((char)(n14 = array[n]))) {
                    ++n;
                    length = 2;
                    n10 = getNorm32FromSurrogatePair(n10, (char)n14);
                }
                else {
                    n14 = '\0';
                    length = 1;
                    n10 = 0L;
                }
                if (nx_contains(set, (char)n11, (char)n14)) {
                    n12 = (cc = 0);
                    extraData = null;
                }
                else if ((n10 & (long)n6) == 0x0L) {
                    n12 = (cc = (int)(0xFFL & n10 >> 8));
                    extraData = null;
                    decompose = -1;
                }
                else {
                    NormalizerImpl.DecomposeArgs o = new NormalizerImpl.DecomposeArgs();
                    decompose = decompose(n10, n6, o);
                    extraData = NormalizerImpl.extraData;
                    length = o.length;
                    cc = o.cc;
                    n12 = o.trailCC;
                    if (length == 1) {
                        n11 = extraData[decompose];
                        n14 = '\0';
                        extraData = null;
                        decompose = -1;
                    }
                }
            }
            if (n3 + length <= n4) {
                if (extraData == null) {
                    if (cc != 0 && cc < n9) {
                        n3 += length;
                        n12 = insertOrdered(array2, n8, n3, n3, (char)n11, (char)n14, cc);
                    }
                    else {
                        array2[n3++] = n11;
                        if (n14 != '\0') {
                            array2[n3++] = n14;
                        }
                    }
                }
                else if (cc != 0 && cc < n9) {
                    n3 += length;
                    n12 = mergeOrdered(array2, n8, n3, extraData, decompose, decompose + length);
                }
                else {
                    do {
                        array2[n3++] = extraData[decompose++];
                    } while (--length > 0);
                }
            }
            else {
                n3 += length;
            }
            n9 = n12;
            if (n9 != 0) {
                continue;
            }
            n8 = n3;
        }
        array3[0] = n9;
        return n3 - n3;
    }
    
    private static int getNextCombining(NormalizerImpl.NextCombiningArgs nextCombiningArgs, int n, UnicodeSet set) {
        nextCombiningArgs.c = nextCombiningArgs.source[nextCombiningArgs.start++];
        long n2 = getNorm32(nextCombiningArgs.c);
        nextCombiningArgs.c2 = '\0';
        nextCombiningArgs.combiningIndex = 0;
        nextCombiningArgs.cc = '\0';
        if ((n2 & 0xFFC0L) == 0x0L) {
            return 0;
        }
        Label_0153: {
            if (!isNorm32Regular(n2)) {
                if (isNorm32HangulOrJamo(n2)) {
                    nextCombiningArgs.combiningIndex = (int)(0xFFFFFFFFL & (0xFFF0L | n2 >> 16));
                    return (int)(n2 & 0xC0L);
                }
                if (nextCombiningArgs.start != n) {
                    char c2 = nextCombiningArgs.source[nextCombiningArgs.start];
                    nextCombiningArgs.c2 = c2;
                    if (UTF16.isTrailSurrogate(c2)) {
                        ++nextCombiningArgs.start;
                        n2 = getNorm32FromSurrogatePair(n2, nextCombiningArgs.c2);
                        break Label_0153;
                    }
                }
                nextCombiningArgs.c2 = '\0';
                return 0;
            }
        }
        if (nx_contains(set, nextCombiningArgs.c, nextCombiningArgs.c2)) {
            return 0;
        }
        nextCombiningArgs.cc = (char)(int)(n2 >> 8 & 0xFFL);
        int n3 = (int)(n2 & 0xC0L);
        if (n3 != 0) {
            int extraDataIndex = getExtraDataIndex(n2);
            nextCombiningArgs.combiningIndex = ((extraDataIndex > 0) ? NormalizerImpl.extraData[extraDataIndex - 1] : '\0');
        }
        return n3;
    }
    
    private static int getCombiningIndexFromStarter(char c, char c2) {
        long n = getNorm32(c);
        if (c2 != '\0') {
            n = getNorm32FromSurrogatePair(n, c2);
        }
        return NormalizerImpl.extraData[getExtraDataIndex(n) - 1];
    }
    
    private static int combine(char[] array, int n, int n2, int[] array2) {
        if (array2.length < 2) {
            throw new IllegalArgumentException();
        }
        int n3;
        while (true) {
            n3 = array[n++];
            if (n3 >= n2) {
                break;
            }
            n += (((array[n] & '耀') != 0x0) ? 2 : 1);
        }
        if ((n3 & '翿') == n2) {
            int n4 = array[n];
            int n5 = (int)(0xFFFFFFFFL & (long)((n4 & '\u2000') + 1));
            int n6;
            int n7;
            if ((n4 & '耀') != 0x0) {
                if ((n4 & '䀀') != 0x0) {
                    n6 = (int)(0xFFFFFFFFL & (long)((n4 & 'Ͽ') | 0xD800));
                    n7 = array[n + 1];
                }
                else {
                    n6 = array[n + 1];
                    n7 = 0;
                }
            }
            else {
                n6 = (n4 & '\u1fff');
                n7 = 0;
            }
            array2[0] = n6;
            array2[1] = n7;
            return n5;
        }
        else {
            return 0;
        }
    }
    
    private static char recompose(NormalizerImpl.RecomposeArgs recomposeArgs, int n, UnicodeSet set) {
        int n2 = 0;
        int n3 = 0;
        int[] array = new int[2];
        int n4 = -1;
        int combiningIndexFromStarter = 0;
        int n5 = 0;
        int cc = '\0';
        NormalizerImpl.NextCombiningArgs nextCombiningArgs = new NormalizerImpl.NextCombiningArgs();
        nextCombiningArgs.source = recomposeArgs.source;
        nextCombiningArgs.cc = '\0';
        nextCombiningArgs.c2 = '\0';
        while (true) {
            nextCombiningArgs.start = recomposeArgs.start;
            int nextCombining = getNextCombining(nextCombiningArgs, recomposeArgs.limit, set);
            int combiningIndex = nextCombiningArgs.combiningIndex;
            recomposeArgs.start = nextCombiningArgs.start;
            Label_0813: {
                if ((nextCombining & 0x80) != 0x0 && n4 != -1) {
                    if ((combiningIndex & 0x8000) != 0x0) {
                        if ((n & 0x100) != 0x0 || cc == '\0') {
                            int start = -1;
                            nextCombining = 0;
                            nextCombiningArgs.c2 = recomposeArgs.source[n4];
                            if (combiningIndex == 65522) {
                                nextCombiningArgs.c2 -= 'ᄀ';
                                if (nextCombiningArgs.c2 < '\u0013') {
                                    start = recomposeArgs.start - 1;
                                    nextCombiningArgs.c = (char)(44032 + (nextCombiningArgs.c2 * 21 + (nextCombiningArgs.c - 4449)) * 28);
                                    if (recomposeArgs.start != recomposeArgs.limit && (nextCombiningArgs.c2 = (char)(recomposeArgs.source[recomposeArgs.start] - 4519)) < '\u001c') {
                                        ++recomposeArgs.start;
                                        nextCombiningArgs.c += nextCombiningArgs.c2;
                                    }
                                    else {
                                        nextCombining = 64;
                                    }
                                    if (!nx_contains(set, (int)nextCombiningArgs.c)) {
                                        recomposeArgs.source[n4] = nextCombiningArgs.c;
                                    }
                                    else {
                                        if (!isHangulWithoutJamoT(nextCombiningArgs.c)) {
                                            --recomposeArgs.start;
                                        }
                                        start = recomposeArgs.start;
                                    }
                                }
                            }
                            else if (isHangulWithoutJamoT(nextCombiningArgs.c2)) {
                                nextCombiningArgs.c2 += nextCombiningArgs.c - 4519;
                                if (!nx_contains(set, (int)nextCombiningArgs.c2)) {
                                    start = recomposeArgs.start - 1;
                                    recomposeArgs.source[n4] = nextCombiningArgs.c2;
                                }
                            }
                            if (start != -1) {
                                for (int k = recomposeArgs.start; k < recomposeArgs.limit; recomposeArgs.source[start++] = recomposeArgs.source[k++]) {}
                                recomposeArgs.start = start;
                                recomposeArgs.limit = start;
                            }
                            nextCombiningArgs.c2 = '\0';
                            if (nextCombining != 0) {
                                if (recomposeArgs.start == recomposeArgs.limit) {
                                    return (char)cc;
                                }
                                combiningIndexFromStarter = 65520;
                                continue;
                            }
                        }
                    }
                    else if ((combiningIndexFromStarter & 0x8000) == 0x0) {
                        if ((n & 0x100) != 0x0) {
                            if (cc == nextCombiningArgs.cc && cc != '\0') {
                                break Label_0813;
                            }
                        }
                        else if (cc >= nextCombiningArgs.cc && cc != '\0') {
                            break Label_0813;
                        }
                        int combine;
                        if (0 != (combine = combine(NormalizerImpl.combiningTable, combiningIndexFromStarter, combiningIndex, array)) && !nx_contains(set, (char)n2, (char)n3)) {
                            n2 = array[0];
                            n3 = array[1];
                            int n6 = (nextCombiningArgs.c2 == '\0') ? (recomposeArgs.start - 1) : (recomposeArgs.start - 2);
                            recomposeArgs.source[n4] = (char)n2;
                            if (n5 != 0) {
                                if (n3 != 0) {
                                    recomposeArgs.source[n4 + 1] = (char)n3;
                                }
                                else {
                                    n5 = 0;
                                    for (int n7 = n4 + 1, i = n7 + 1; i < n6; recomposeArgs.source[n7++] = recomposeArgs.source[i++]) {}
                                    --n6;
                                }
                            }
                            else if (n3 != 0) {
                                n5 = 1;
                                recomposeArgs.source[n4 + 1] = (char)n3;
                            }
                            if (n6 < recomposeArgs.start) {
                                for (int j = recomposeArgs.start; j < recomposeArgs.limit; recomposeArgs.source[n6++] = recomposeArgs.source[j++]) {}
                                recomposeArgs.start = n6;
                                recomposeArgs.limit = n6;
                            }
                            if (recomposeArgs.start == recomposeArgs.limit) {
                                return (char)cc;
                            }
                            if (combine > 1) {
                                combiningIndexFromStarter = getCombiningIndexFromStarter((char)n2, (char)n3);
                                continue;
                            }
                            n4 = -1;
                            continue;
                        }
                    }
                }
            }
            cc = nextCombiningArgs.cc;
            if (recomposeArgs.start == recomposeArgs.limit) {
                return (char)cc;
            }
            if (nextCombiningArgs.cc == '\0') {
                if ((nextCombining & 0x40) != 0x0) {
                    if (nextCombiningArgs.c2 == '\0') {
                        n5 = 0;
                        n4 = recomposeArgs.start - 1;
                    }
                    else {
                        n5 = 0;
                        n4 = recomposeArgs.start - 2;
                    }
                    combiningIndexFromStarter = combiningIndex;
                }
                else {
                    n4 = -1;
                }
            }
            else {
                if ((n & 0x2000) == 0x0) {
                    continue;
                }
                n4 = -1;
            }
        }
    }
    
    private static int findPreviousStarter(char[] src, int start, int current, int n, int n2, char c) {
        NormalizerImpl.PrevArgs prevArgs = new NormalizerImpl.PrevArgs();
        prevArgs.src = src;
        prevArgs.start = start;
        prevArgs.current = current;
        while (prevArgs.start < prevArgs.current && !isTrueStarter(getPrevNorm32(prevArgs, (int)c, n | n2), n, n2)) {}
        return prevArgs.current;
    }
    
    private static int findNextStarter(char[] array, int i, int n, int n2, int n3, char c) {
        int n4 = 0xFF00 | n2;
        NormalizerImpl.DecomposeArgs decomposeArgs = new NormalizerImpl.DecomposeArgs();
        while (i != n) {
            int n5 = array[i];
            if (n5 >= c) {
                long n6 = getNorm32((char)n5);
                if ((n6 & (long)n4) != 0x0L) {
                    int n7;
                    if (isNorm32LeadSurrogate(n6)) {
                        if (i + 1 != n) {
                            if (!UTF16.isTrailSurrogate((char)(n7 = array[i + 1]))) {
                                break;
                            }
                            n6 = getNorm32FromSurrogatePair(n6, (char)n7);
                            if ((n6 & (long)n4) == 0x0L) {
                                break;
                            }
                        }
                        else {
                            break;
                        }
                    }
                    else {
                        n7 = '\0';
                    }
                    if ((n6 & (long)n3) != 0x0L) {
                        int decompose = decompose(n6, n3, decomposeArgs);
                        if (decomposeArgs.cc == 0 && (getNorm32(NormalizerImpl.extraData, decompose, n2) & (long)n2) == 0x0L) {
                            break;
                        }
                    }
                    i += ((n7 == '\0') ? 1 : 2);
                    continue;
                }
            }
            return i;
        }
        return i;
    }
    
    private static char[] composePart(NormalizerImpl.ComposePartArgs composePartArgs, int n, char[] array, int n2, int n3, int n4, UnicodeSet set) {
        int n5 = (n4 & 0x1000) != 0x0;
        int[] array2 = { 0 };
        char[] source = new char[(n3 - n) * 20];
        while (true) {
            composePartArgs.length = decompose(array, n, n2, source, 0, source.length, (boolean)n5, array2, set);
            if (composePartArgs.length <= source.length) {
                break;
            }
            source = new char[composePartArgs.length];
        }
        int n6 = composePartArgs.length;
        if (composePartArgs.length >= 2) {
            NormalizerImpl.RecomposeArgs o = new NormalizerImpl.RecomposeArgs();
            o.source = source;
            o.start = 0;
            o.limit = n6;
            composePartArgs.prevCC = recompose(o, n4, set);
            n6 = o.limit;
        }
        composePartArgs.length = n6;
        return source;
    }
    
    private static boolean composeHangul(char c, char c2, long norm32, char[] array, int[] array2, int n, boolean b, char[] array3, int n2, UnicodeSet set) {
        int n3 = array2[0];
        if (isJamoVTNorm32JamoV(norm32)) {
            int n5 = (char)(c - 4352);
            if (n5 < '\u0013') {
                int n6 = (char)(44032 + (n5 * 21 + (c2 - 4449)) * 28);
                if (n3 != n) {
                    int n7 = array[n3];
                    int n8;
                    if ((n8 = (char)(n7 - 4519)) < '\u001c') {
                        ++n3;
                        n6 = (char)(n6 + n8);
                    }
                    else if (b) {
                        norm32 = getNorm32((char)n7);
                        if (isNorm32Regular(norm32) && (norm32 & 0x8L) != 0x0L) {
                            NormalizerImpl.DecomposeArgs decomposeArgs = new NormalizerImpl.DecomposeArgs();
                            int decompose = decompose(norm32, 8, decomposeArgs);
                            int n9;
                            if (decomposeArgs.length == 1 && (n9 = (char)(NormalizerImpl.extraData[decompose] - 4519)) < '\u001c') {
                                ++n3;
                                n6 = (char)(n6 + n9);
                            }
                        }
                    }
                }
                if (nx_contains(set, n6)) {
                    if (!isHangulWithoutJamoT((char)n6)) {
                        --n3;
                    }
                    return false;
                }
                else {
                    array3[n2] = n6;
                    array2[0] = n3;
                    return true;
                }
            }
        }
        else if (isHangulWithoutJamoT(c)) {
            int n4 = (char)(c + (c2 - 4519));
            if (nx_contains(set, n4)) {
                return false;
            }
            array3[n2] = n4;
            array2[0] = n3;
            return true;
        }
        return false;
    }
    
    public static int compose(char[] array, int nextStarter, int n, char[] array2, int n2, int n3, int n4, UnicodeSet set) {
        int[] array3 = { 0 };
        int n5;
        int n6;
        if ((n4 & 0x1000) != 0x0) {
            n5 = (char)NormalizerImpl.indexes[7];
            n6 = 34;
        }
        else {
            n5 = (char)NormalizerImpl.indexes[6];
            n6 = 17;
        }
        int n7 = nextStarter;
        int n8 = 0xFF00 | n6;
        int n9 = 0;
        int prevCC = 0;
        long n10 = 0L;
        int n11 = '\0';
        while (true) {
            for (; nextStarter != n && ((n11 = array[nextStarter]) < n5 || ((n10 = getNorm32((char)n11)) & (long)n8) == 0x0L); ++nextStarter) {
                prevCC = 0;
            }
            if (nextStarter != nextStarter) {
                int n12 = nextStarter - nextStarter;
                if (n2 + n12 <= n3) {
                    System.arraycopy(array, nextStarter, array2, n2, n12);
                }
                n2 = (n9 = n2 + n12);
                n7 = nextStarter - 1;
                if (UTF16.isTrailSurrogate(array[n7]) && nextStarter < n7 && UTF16.isLeadSurrogate(array[n7 - 1])) {
                    --n7;
                }
            }
            if (nextStarter == n) {
                break;
            }
            ++nextStarter;
            int n13;
            int length;
            int n15;
            if (isNorm32HangulOrJamo(n10)) {
                n15 = (prevCC = 0);
                n9 = n2;
                array3[0] = nextStarter;
                if (n2 > 0 && composeHangul(array[nextStarter - 1], (char)n11, n10, array, array3, n, (n4 & 0x1000) != 0x0, array2, (n2 <= n3) ? (n2 - 1) : 0, set)) {
                    nextStarter = (n7 = array3[0]);
                    continue;
                }
                nextStarter = array3[0];
                n13 = '\0';
                length = 1;
                n7 = nextStarter;
            }
            else {
                if (isNorm32Regular(n10)) {
                    n13 = '\0';
                    length = 1;
                }
                else if (nextStarter != n && UTF16.isTrailSurrogate((char)(n13 = array[nextStarter]))) {
                    ++nextStarter;
                    length = 2;
                    n10 = getNorm32FromSurrogatePair(n10, (char)n13);
                }
                else {
                    n13 = '\0';
                    length = 1;
                    n10 = 0L;
                }
                NormalizerImpl.ComposePartArgs o = new NormalizerImpl.ComposePartArgs();
                if (nx_contains(set, (char)n11, (char)n13)) {
                    n15 = 0;
                }
                else if ((n10 & (long)n6) == 0x0L) {
                    n15 = (int)(0xFFL & n10 >> 8);
                }
                else {
                    int n14 = n6 << 2 & 0xF;
                    if (isTrueStarter(n10, 0xFF00 | n6, n14)) {
                        n7 = nextStarter;
                    }
                    else {
                        n2 -= nextStarter - n7;
                    }
                    nextStarter = findNextStarter(array, nextStarter, n, n6, n14, (char)n5);
                    o.prevCC = prevCC;
                    o.length = length;
                    char[] composePart = composePart(o, n7, array, nextStarter, n, n4, set);
                    if (composePart == null) {
                        break;
                    }
                    prevCC = o.prevCC;
                    int length2 = o.length;
                    if (n2 + o.length <= n3) {
                        for (int i = 0; i < o.length; array2[n2++] = composePart[i++], --length2) {}
                    }
                    else {
                        n2 += length2;
                    }
                    n7 = nextStarter;
                    continue;
                }
            }
            if (n2 + length <= n3) {
                if (n15 != 0 && n15 < prevCC) {
                    n2 += length;
                    prevCC = insertOrdered(array2, n9, n2, n2, (char)n11, (char)n13, n15);
                }
                else {
                    array2[n2++] = n11;
                    if (n13 != '\0') {
                        array2[n2++] = n13;
                    }
                    prevCC = n15;
                }
            }
            else {
                n2 += length;
                prevCC = n15;
            }
        }
        return n2 - n2;
    }
    
    public static int getCombiningClass(int n) {
        return (int)(getNorm32(n) >> 8 & 0xFFL);
    }
    
    public static boolean isFullCompositionExclusion(int n) {
        return NormalizerImpl.isFormatVersion_2_1 && (NormalizerImpl.AuxTrieImpl.auxTrie.getCodePointValue(n) & 'Ѐ') != 0x0;
    }
    
    public static boolean isCanonSafeStart(int n) {
        return NormalizerImpl.isFormatVersion_2_1 && (NormalizerImpl.AuxTrieImpl.auxTrie.getCodePointValue(n) & 'ࠀ') == 0x0;
    }
    
    public static boolean isNFSkippable(int n, NormalizerBase.Mode mode, long n2) {
        n2 &= 0xFFFFFFFFL;
        long norm32 = getNorm32(n);
        if ((norm32 & n2) != 0x0L) {
            return false;
        }
        if (mode == NormalizerBase.NFD || mode == NormalizerBase.NFKD || mode == NormalizerBase.NONE) {
            return true;
        }
        if ((norm32 & 0x4L) == 0x0L) {
            return true;
        }
        if (isNorm32HangulOrJamo(norm32)) {
            return !isHangulWithoutJamoT((char)n);
        }
        return NormalizerImpl.isFormatVersion_2_2 && ((long)NormalizerImpl.AuxTrieImpl.auxTrie.getCodePointValue(n) & 0x1000L) == 0x0L;
    }
    
    public static UnicodeSet addPropertyStarts(UnicodeSet set) {
        TrieIterator trieIterator = new TrieIterator(NormalizerImpl.NormTrieImpl.normTrie);
        RangeValueIterator.Element element = new RangeValueIterator.Element();
        while (trieIterator.next(element)) {
            set.add(element.start);
        }
        TrieIterator trieIterator2 = new TrieIterator(NormalizerImpl.FCDTrieImpl.fcdTrie);
        RangeValueIterator.Element element2 = new RangeValueIterator.Element();
        while (trieIterator2.next(element2)) {
            set.add(element2.start);
        }
        if (NormalizerImpl.isFormatVersion_2_1) {
            TrieIterator trieIterator3 = new TrieIterator(NormalizerImpl.AuxTrieImpl.auxTrie);
            RangeValueIterator.Element element3 = new RangeValueIterator.Element();
            while (trieIterator3.next(element3)) {
                set.add(element3.start);
            }
        }
        for (int i = 44032; i < 55204; i += 28) {
            set.add(i);
            set.add(i + 1);
        }
        set.add(55204);
        return set;
    }
    
    public static final int quickCheck(int n, int n2) {
        int n3 = (int)getNorm32(n) & (new int[] { 0, 0, 4, 8, 17, 34 })[n2];
        if (n3 == 0) {
            return 1;
        }
        if ((n3 & 0xF) != 0x0) {
            return 0;
        }
        return 2;
    }
    
    private static int strCompare(char[] array, int i, int n, char[] array2, int n2, int n3, boolean b) {
        int n4 = n - i;
        int n5 = n3 - n2;
        int n6;
        int n7;
        if (n4 < n5) {
            n6 = -1;
            n7 = i + n4;
        }
        else if (n4 == n5) {
            n6 = 0;
            n7 = i + n4;
        }
        else {
            n6 = 1;
            n7 = i + n5;
        }
        if (array == array2) {
            return n6;
        }
        while (i != n7) {
            int n8 = array[i];
            int n9 = array2[n2];
            if (n8 != n9) {
                int n10 = i + n4;
                int n11 = n2 + n5;
                if (n8 >= '?' && n9 >= '?' && b) {
                    if ((n8 > '?' || i + 1 == n10 || !UTF16.isTrailSurrogate(array[i + 1])) && (!UTF16.isTrailSurrogate((char)n8) || i == i || !UTF16.isLeadSurrogate(array[i - 1]))) {
                        n8 = (char)(n8 - 10240);
                    }
                    if ((n9 > '?' || n2 + 1 == n11 || !UTF16.isTrailSurrogate(array2[n2 + 1])) && (!UTF16.isTrailSurrogate((char)n9) || n2 == n2 || !UTF16.isLeadSurrogate(array2[n2 - 1]))) {
                        n9 = (char)(n9 - 10240);
                    }
                }
                return n8 - n9;
            }
            else {
                ++i;
                ++n2;
            }
        }
        return n6;
    }
    
    private static final synchronized UnicodeSet internalGetNXHangul() {
        if (NormalizerImpl.nxCache[1] == null) {
            NormalizerImpl.nxCache[1] = new UnicodeSet(44032, 55203);
        }
        return NormalizerImpl.nxCache[1];
    }
    
    private static final synchronized UnicodeSet internalGetNXCJKCompat() {
        if (NormalizerImpl.nxCache[2] == null) {
            UnicodeSet set = new UnicodeSet("[:Ideographic:]");
            UnicodeSet set2 = new UnicodeSet();
            UnicodeSetIterator unicodeSetIterator = new UnicodeSetIterator(set);
            while (unicodeSetIterator.nextRange() && unicodeSetIterator.codepoint != UnicodeSetIterator.IS_STRING) {
                for (int i = unicodeSetIterator.codepoint; i <= unicodeSetIterator.codepointEnd; ++i) {
                    if ((getNorm32(i) & 0x4L) > 0L) {
                        set2.add(i);
                    }
                }
            }
            NormalizerImpl.nxCache[2] = set2;
        }
        return NormalizerImpl.nxCache[2];
    }
    
    private static final synchronized UnicodeSet internalGetNXUnicode(int n) {
        n &= 0xE0;
        if (n == 0) {
            return null;
        }
        if (NormalizerImpl.nxCache[n] == null) {
            UnicodeSet set = new UnicodeSet();
            switch (n) {
                case 32: {
                    set.applyPattern("[:^Age=3.2:]");
                    NormalizerImpl.nxCache[n] = set;
                    break;
                }
                default: {
                    return null;
                }
            }
        }
        return NormalizerImpl.nxCache[n];
    }
    
    private static final synchronized UnicodeSet internalGetNX(int n) {
        n &= 0xFF;
        if (NormalizerImpl.nxCache[n] == null) {
            if (n == 1) {
                return internalGetNXHangul();
            }
            if (n == 2) {
                return internalGetNXCJKCompat();
            }
            if ((n & 0xE0) != 0x0 && (n & 0x1F) == 0x0) {
                return internalGetNXUnicode(n);
            }
            UnicodeSet set = new UnicodeSet();
            UnicodeSet internalGetNXHangul;
            if ((n & 0x1) != 0x0 && null != (internalGetNXHangul = internalGetNXHangul())) {
                set.addAll(internalGetNXHangul);
            }
            UnicodeSet internalGetNXCJKCompat;
            if ((n & 0x2) != 0x0 && null != (internalGetNXCJKCompat = internalGetNXCJKCompat())) {
                set.addAll(internalGetNXCJKCompat);
            }
            UnicodeSet internalGetNXUnicode;
            if ((n & 0xE0) != 0x0 && null != (internalGetNXUnicode = internalGetNXUnicode(n))) {
                set.addAll(internalGetNXUnicode);
            }
            NormalizerImpl.nxCache[n] = set;
        }
        return NormalizerImpl.nxCache[n];
    }
    
    public static final UnicodeSet getNX(int n) {
        if ((n &= 0xFF) == 0x0) {
            return null;
        }
        return internalGetNX(n);
    }
    
    private static final boolean nx_contains(UnicodeSet set, int n) {
        return set != null && set.contains(n);
    }
    
    private static final boolean nx_contains(UnicodeSet set, char c, char c2) {
        return set != null && set.contains((c2 == '\0') ? c : UCharacterProperty.getRawSupplementary(c, c2));
    }
    
    public static int getDecompose(int[] array, String[] array2) {
        NormalizerImpl.DecomposeArgs decomposeArgs = new NormalizerImpl.DecomposeArgs();
        int n = -1;
        int n2 = 0;
        while (++n < 195102) {
            if (n == 12543) {
                n = 63744;
            }
            else if (n == 65536) {
                n = 119134;
            }
            else if (n == 119233) {
                n = 194560;
            }
            long norm32 = getNorm32(n);
            if ((norm32 & 0x4L) == 0x0L || n2 >= array.length) {
                continue;
            }
            array[n2] = n;
            array2[n2++] = new String(NormalizerImpl.extraData, decompose(norm32, decomposeArgs), decomposeArgs.length);
        }
        return n2;
    }
    
    private static boolean needSingleQuotation(char c) {
        return (c >= '\t' && c <= '\r') || (c >= ' ' && c <= '/') || (c >= ':' && c <= '@') || (c >= '[' && c <= '`') || (c >= '{' && c <= '~');
    }
    
    public static String canonicalDecomposeWithSingleQuotation(String s) {
        char[] charArray = s.toCharArray();
        int n = 0;
        int length = charArray.length;
        char[] value = new char[charArray.length * 3];
        int count = 0;
        int n2 = value.length;
        char[] array = new char[3];
        int n3 = 4;
        int n4 = (char)NormalizerImpl.indexes[8];
        int n5 = 0xFF00 | n3;
        int n6 = 0;
        int n7 = 0;
        long n8 = 0L;
        int n9 = '\0';
        while (true) {
            for (; n != length && ((n9 = charArray[n]) < n4 || ((n8 = getNorm32((char)n9)) & (long)n5) == 0x0L || (n9 >= '가' && n9 <= '힣')); ++n) {
                n7 = 0;
            }
            if (n != n) {
                int n10 = n - n;
                if (count + n10 <= n2) {
                    System.arraycopy(charArray, n, value, count, n10);
                }
                count = (n6 = count + n10);
            }
            if (n == length) {
                break;
            }
            ++n;
            int n11;
            int length2;
            if (isNorm32Regular(n8)) {
                n11 = '\0';
                length2 = 1;
            }
            else if (n != length && Character.isLowSurrogate((char)(n11 = charArray[n]))) {
                ++n;
                length2 = 2;
                n8 = getNorm32FromSurrogatePair(n8, (char)n11);
            }
            else {
                n11 = '\0';
                length2 = 1;
                n8 = 0L;
            }
            int decompose;
            char[] extraData;
            int cc;
            if ((n8 & (long)n3) == 0x0L) {
                int n12 = cc = (int)(0xFFL & n8 >> 8);
                extraData = null;
                decompose = -1;
            }
            else {
                NormalizerImpl.DecomposeArgs o = new NormalizerImpl.DecomposeArgs();
                decompose = decompose(n8, n3, o);
                extraData = NormalizerImpl.extraData;
                length2 = o.length;
                cc = o.cc;
                int n12 = o.trailCC;
                if (length2 == 1) {
                    n9 = extraData[decompose];
                    n11 = '\0';
                    extraData = null;
                    decompose = -1;
                }
            }
            if (count + length2 * 3 >= n2) {
                char[] array2 = new char[n2 * 2];
                System.arraycopy(value, 0, array2, 0, count);
                value = array2;
                n2 = value.length;
            }
            int n12;
            if (extraData == null) {
                if (needSingleQuotation((char)n9)) {
                    value[count++] = '\'';
                    value[count++] = n9;
                    value[count++] = '\'';
                    n12 = 0;
                }
                else if (cc != 0 && cc < n7) {
                    count += length2;
                    n12 = insertOrdered(value, n6, count, count, (char)n9, (char)n11, cc);
                }
                else {
                    value[count++] = n9;
                    if (n11 != '\0') {
                        value[count++] = n11;
                    }
                }
            }
            else if (needSingleQuotation(extraData[decompose])) {
                value[count++] = '\'';
                value[count++] = extraData[decompose++];
                value[count++] = '\'';
                --length2;
                do {
                    value[count++] = extraData[decompose++];
                } while (--length2 > 0);
            }
            else if (cc != 0 && cc < n7) {
                count += length2;
                n12 = mergeOrdered(value, n6, count, extraData, decompose, decompose + length2);
            }
            else {
                do {
                    value[count++] = extraData[decompose++];
                } while (--length2 > 0);
            }
            n7 = n12;
            if (n7 != 0) {
                continue;
            }
            n6 = count;
        }
        return new String(value, 0, count);
    }
    
    public static String convert(String s) {
        if (s == null) {
            return null;
        }
        StringBuffer sb = new StringBuffer();
        int nextCodePoint;
        while ((nextCodePoint = UCharacterIterator.getInstance(s).nextCodePoint()) != -1) {
            switch (nextCodePoint) {
                case 194664: {
                    sb.append(NormalizerImpl.corrigendum4MappingTable[0]);
                    continue;
                }
                case 194676: {
                    sb.append(NormalizerImpl.corrigendum4MappingTable[1]);
                    continue;
                }
                case 194847: {
                    sb.append(NormalizerImpl.corrigendum4MappingTable[2]);
                    continue;
                }
                case 194911: {
                    sb.append(NormalizerImpl.corrigendum4MappingTable[3]);
                    continue;
                }
                case 195007: {
                    sb.append(NormalizerImpl.corrigendum4MappingTable[4]);
                    continue;
                }
                default: {
                    UTF16.append(sb, nextCodePoint);
                    continue;
                }
            }
        }
        return sb.toString();
    }
    
    static {
        try {
            IMPL = new NormalizerImpl();
        }
        catch (Exception ex) {
            throw new RuntimeException(ex.getMessage());
        }
        nxCache = new UnicodeSet[256];
        corrigendum4MappingTable = new char[][] { { '?', '?' }, { '弳' }, { '䎫' }, { '窮' }, { '䵗' } };
    }
    
    static final class NormTrieImpl implements Trie.DataManipulate
    {
        static IntTrie normTrie;
        
        @Override
        public int getFoldingOffset(int n) {
            return 2048 + (n >> 11 & 0x7FE0);
        }
        
        static {
            NormalizerImpl.NormTrieImpl.normTrie = null;
        }
    }
    
    static final class FCDTrieImpl implements Trie.DataManipulate
    {
        static CharTrie fcdTrie;
        
        @Override
        public int getFoldingOffset(int n) {
            return n;
        }
        
        static {
            NormalizerImpl.FCDTrieImpl.fcdTrie = null;
        }
    }
    
    static final class AuxTrieImpl implements Trie.DataManipulate
    {
        static CharTrie auxTrie;
        
        @Override
        public int getFoldingOffset(int n) {
            return (n & 0x3FF) << 5;
        }
        
        static {
            NormalizerImpl.AuxTrieImpl.auxTrie = null;
        }
    }
    
    private static final class DecomposeArgs
    {
        int cc;
        int trailCC;
        int length;
    }
    
    private static final class NextCCArgs
    {
        char[] source;
        int next;
        int limit;
        char c;
        char c2;
    }
    
    private static final class PrevArgs
    {
        char[] src;
        int start;
        int current;
        char c;
        char c2;
    }
    
    private static final class NextCombiningArgs
    {
        char[] source;
        int start;
        char c;
        char c2;
        int combiningIndex;
        char cc;
    }
    
    private static final class RecomposeArgs
    {
        char[] source;
        int start;
        int limit;
    }
    
    private static final class ComposePartArgs
    {
        int prevCC;
        int length;
    }
}
